IRActorBehavior subclass: #ConcurrentActorBehavior
	instanceVariableNames: 'synchroMutualExclusion '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Concurrent'!


!ConcurrentActorBehavior methodsFor: 'initialize'!

actalkPrivateInitialize
	"Initialize the semaphore for mutual exclusion of synchronization actions."

	super actalkPrivateInitialize.
	synchroMutualExclusion := Semaphore forMutualExclusion! !

!ConcurrentActorBehavior methodsFor: 'subprocess setting'!

behaviorSubProcessBlockFor: aMessage
	"Create and return the subprocess block to consume a message."
	"No default behavior. Depend on the semantics of the synchronization."

	^self subclassResponsability!

setSubProcessFor: aMessage
	"Create and start a new subprocess to consume a message."
	"The message to be consumed is passed as an argument.
	This allowes lazy creation of subprocesses when needed (message exists)."

	(self behaviorSubProcessBlockFor: aMessage) newProcess resume!

subProcessAcceptMessage: aMessage
	"Acceptance of a message by a subprocess."
	"Default is standard acceptMessage: as for a primary process.
	Note we use super in order to freeze the definition of accept (as in class Basic2ActorBehavior).
	This is because method acceptMessage: will be redefined in subclasses
	in order to specify the behavior of the primary process."

	super acceptMessage: aMessage! !

!ConcurrentActorBehavior methodsFor: 'events'!

kernelEventAccept: aMessage
	"Dispatch onto synchronization events which should be mutually exclusive."

	super kernelEventAccept: aMessage.
	synchroMutualExclusion critical:
		[self synchroAccept: aMessage]!

kernelEventComplete: aMessage
	"Dispatch onto synchronization events which should be mutually exclusive."

	super kernelEventComplete: aMessage.
	synchroMutualExclusion critical:
		[self synchroComplete: aMessage]!

kernelEventReceive: aMessage
	"Dispatch onto synchronization events which should be mutually exclusive."

	super kernelEventReceive: aMessage.
	synchroMutualExclusion critical:
		[self synchroReceive: aMessage]! !

!ConcurrentActorBehavior methodsFor: 'synchro events'!

synchroAccept: aMessage
	"This synchronization event method is called when the behavior accepts a message.
	It is used for synchronization purposes, for instance updating synchronization counters.
	At this abstract level, default behavior is empty."!

synchroComplete: aMessage
	"This synchronization event method is called once the behavior has completed computation of a message.
	It is used for synchronization purposes, for instance updating synchronization counters.
	At this abstract level, default behavior is empty."!

synchroReceive: aMessage
	"This synchronization event method is called once the behavior has received a message.
	It is used for synchronization purposes, for instance updating synchronization counters.
	At this abstract level, default behavior is empty."! !
