AbclActorBehavior subclass: #Sender
	instanceVariableNames: 'receiver returnedValue delay '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
Sender comment:
'Classes Sender and Receiver are used to test various communication and synchronization patterns between actors.
Check the examples of class Sender.'!


!Sender methodsFor: 'initialize'!

initialize

	receiver := Receiver new actor.
	delay := 8! !

!Sender methodsFor: 'script'!

futureWithoutWait

	Transcript show: 'Sender sends a future message to the receiver...'; cr.
	returnedValue := receiver asynchronousSendAndReplyTo: #future.
	Transcript show: 'The future is now: ' , returnedValue printString; cr.
	self sleep: delay.
	Transcript show: 'The future is now: ' , returnedValue printString; cr.
	Transcript show: 'The future accepted the + message: ' , (returnedValue + 1) printString; cr!

futureWithWait

	Transcript show: 'Sender sends a future message to the receiver...'; cr.
	returnedValue := receiver asynchronousSendAndReplyTo: #future.
	Transcript show: 'Sender wants to add 1 to the future: ' , returnedValue printString; cr.
	Transcript show: 'The future accepted the + message: ' , (returnedValue + 1) printString; cr!

mAfuture

	Transcript show: 'Sender sends an MA future message to the receiver...'; cr.
	returnedValue := receiver asynchronousSendAndRepliesTo: #MAfuture.
	self sleep: delay.
	Transcript show: 'Sender will retrieve values from the future: ' , returnedValue printString; cr.
	Transcript show: 'First value is: ' , returnedValue next printString; cr.
	Transcript show: 'Second value is: ' , returnedValue next printString; cr!

now

	Transcript show: 'Sender sends a now message to the receiver and waits for the returned value...'; cr.
	returnedValue := receiver asynchronousSendAndReplyTo: #now.
	Transcript show: 'Sender received the value and adds 1 to it: ' , (returnedValue + 1) printString; cr!

past

	Transcript show: 'Sender sends a past message to the receiver...'; cr.
	receiver asynchronousSendAndReplyTo: Printer new actor!

smalltalkCall

	Transcript show: 'Sender sends a Smalltalk message to the receiver and waits for the returned value...'; cr.
	returnedValue := receiver asObject smalltalkSend.
	Transcript show: 'Sender received the value and adds 1 to it: ' , (returnedValue + 1) printString; cr! !

!Sender methodsFor: 'private routines'!

sleep: time

	Transcript show: 'Sender sleeps (s) for ' , time printString , ' seconds'; cr.
	time timesRepeat:
		[(Delay forSeconds: 1) wait.
		Transcript show: 's'].
	Transcript cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Sender class
	instanceVariableNames: ''!


!Sender class methodsFor: 'instance creation'!

new
	^super new initialize! !

!Sender class methodsFor: 'example'!

example
	"Examples to check synchronization between sender and receiver
	through various communication patterns.
	Try one of these:

	Sender new actor smalltalkCall.
	Sender new actor past.
	Sender new actor now.
	Sender new actor futureWithoutWait.
	Sender new actor futureWithWait.
	Sender new actor mAfuture.

	"! !

AbclActorBehavior subclass: #Receiver
	instanceVariableNames: 'valueToReturn delay '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
Receiver comment:
'Classes Sender and Receiver are used to test various communication and synchronization patterns between actors.
Check the examples of class Sender.'!


!Receiver methodsFor: 'initialize'!

initialize

	delay := 5.
	valueToReturn := 100! !

!Receiver methodsFor: 'script'!

asynchronousSendAndRepliesTo: r
	"Compute and send two replies to the MAfuture type reply destination."

	self compute: delay.
	Transcript show: 'Receiver returns first: ' , valueToReturn printString , ' to ' , r printString; cr.
	r reply: valueToReturn.
	self compute: delay.
	Transcript show: 'Receiver returns second: ' , (valueToReturn + 1) printString , ' to ' , r printString; cr.
	r reply: valueToReturn + 1!

asynchronousSendAndReplyTo: r
	"Compute and send one reply to the reply destination."

	self compute: delay.
	Transcript show: 'Receiver returns: ' , valueToReturn printString , ' to ' , r printString; cr.
	r reply: valueToReturn!

smalltalkSend
	"Compute and return a value."

	self compute.
	Transcript show: 'Receiver returns: ' , valueToReturn printString; cr.
	^valueToReturn! !

!Receiver methodsFor: 'private routines'!

compute: time

	Transcript show: 'Receiver computes (r) for ' , time printString , ' seconds'; cr.
	time timesRepeat:
		[(Delay forSeconds: 1) wait.
		Transcript show: 'r'].
	Transcript cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Receiver class
	instanceVariableNames: ''!


!Receiver class methodsFor: 'instance creation'!

new

	^super new initialize! !

AbclActorBehavior subclass: #AbclFibonacci
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
AbclFibonacci comment:
'Class AbclFibonacci implements fibonacci computation with two variants.

The first one is using two waitFor:andDo: constructs to wait for both subresults, as proposed in Abcl/R2 (Masuhara et al. 1993).

The second one is adding two future type transmissions, as proposed in Abcl/f (Taura et al. 1994).

Note that in the case of future type message passing, future objects are returned.
If the value is not computed yet for the receiver of the + operation, + implicitly waits for the assignment of the future (see class Future). But if the receiver is computed but the argument is not computed yet, + operation will fail (because it expects a numeric argument, not a future). Meanwhile, such a failure exchanges argument with receiver and retries to perform the operation. So eventually the result is to be completed with no failure.'!


!AbclFibonacci methodsFor: 'script'!

n2: n replyTo: r
	"Second version:
		Create recursively two new actors to handle the two recursive sub computations.
		Send two future type messages and sum up their values."

	n < 2
		ifTrue: [r reply: 1]
		ifFalse:
			[r reply:
				(self class new actor n: n-1 replyTo: #future)
				+ (self class new actor n: n-2 replyTo: #future)]!

n: n replyTo: r
	"First version:
		Create recursively two new actors to handle the two recursive sub computations.
		Wait for both of their replies."

	n < 2
		ifTrue: [r reply: 1]
		ifFalse:
			[self class new actor n: n-1 replyTo: aself.
			self class new actor n: n-2 replyTo:aself.
			self waitFor: #(reply:) andDo:
				[:v1 | self waitFor: #(reply:) andDo:
					[:v2 | r reply: v1 + v2]]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbclFibonacci class
	instanceVariableNames: ''!


!AbclFibonacci class methodsFor: 'example'!

example
	"AbclFibonacci example"

	"First version:
		Create recursively two new actors to handle the two recursive sub computations.
		Wait for both of their replies."

	AbclFibonacci new actor
		n: 10 replyTo: Print!

example2
	"AbclFibonacci example2"

	"Second version:
		Create recursively two new actors to handle the two recursive sub computations.
		Send two future type messages and sum up their values."

	AbclFibonacci new actor
		n2: 10 replyTo: Print! !

ExpressAbclActorBehavior subclass: #TreeExtractor
	instanceVariableNames: 'output number '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
TreeExtractor comment:
'See comment of class TreeComparator.'!


!TreeExtractor methodsFor: 'initialize'!

number: anInteger

	number := anInteger! !

!TreeExtractor methodsFor: 'script'!

ExpressStop
	"This is an express mode message."
	"Interrupt and abort current computation (message fringe:replyTo:)."

	Transcript show: self printString , ' is stopped.'; cr.
	self nonResume!

fringe: tree replyTo: aReplyDestination
	"Initialize and start extraction of elements from the tree."

	output := aReplyDestination.	"Store the reply destination, a MAFuture,"
								"used as a (permanent) communication channel."
	Transcript show: self printString , ' extract: ' , tree printString; cr.
	self extract: tree.
	output reply: #EOT	"EOT = end of tree special symbol."		"Sent after completion."! !

!TreeExtractor methodsFor: 'private routines'!

extract: tree

	Processor yield.		"To simulate simultaneity of computing between the two extractors.
						Note that event handling methods are not effective because this is a private routine."
	"Below is some standard traversal of a tree, except that it is represented as an array.
	See protocol tree simulation in class Array."
	(tree isKindOf: Array)
		ifTrue:		"A non leaf, that is a tree."
			[tree isEmpty	
				ifFalse: [self extract: tree left;
							extract: tree right]]
		ifFalse:		"A leaf."
			[Transcript show: self printString , ' : ' , tree printString; cr.
			output reply: tree]		"Return it to the comparator."! !

!TreeExtractor methodsFor: 'printing'!

printOn: aStream

	aStream nextPutAll: self class name , number printString! !

ExpressAbclActorBehavior subclass: #ExpressCheck
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
ExpressCheck comment:
'Class ExpressCheck is a simple example to show how express messages may interrupt standard message computation.
Note that because method Express does a nonResume, if a normal message (normal) was interrupted, it is aborted.'!


!ExpressCheck methodsFor: 'script'!

Express: delay
	"This is an express mode message."

	Transcript show: 'start express'; cr.
	(Delay forMilliseconds: delay) wait.
	Transcript show: 'express stop'; cr!

ExpressNonResume: delay
	"This is an express mode message."

	Transcript show: 'nonResume '.
	self Express: delay.
	self nonResume!

normal: delay

	Transcript show: 'start normal'; cr.
	(Delay forMilliseconds: delay) wait.
	Transcript show: 'normal stop'; cr!

normalAtomic: delay

	self expressAtomic:
		[Transcript show: 'atomic '.
		self normal: delay]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ExpressCheck class
	instanceVariableNames: ''!


!ExpressCheck class methodsFor: 'example'!

example
	"ExpressCheck example"

	| actor |
	actor := ExpressCheck new actor.
	actor normal: 100; ExpressNonResume: 100; normalAtomic: 100; normal: 100.
	(Delay forMilliseconds: 1500) wait.
	actor Express: 40! !

AbclActorBehavior subclass: #TreeComparator
	instanceVariableNames: 'extractor1 extractor2 input1 input2 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
TreeComparator comment:
'TreeComparator implements pattern matching of the fringes of two trees (implemented as two Smalltalk arrays).
(This is the classical same-fringe example). This Abcl/1 version was originally described in OOPSLA''86 paper on Abcl/1. It was some derivation of algorithm introduced by Bernard Serpette in his thesis (1984) on concurrent Lisp (named Pive) with Unix-like pipes were used.

There are three actors in the algorithm.
Two extractors (TreeExtractor) extract the fringe of each tree, that is generates successive leaves.
The comparator (TreeComparator) compares the two fringes leaf by leaf.
The comparator send the initial requests to both extractors through two MAfuture type messages.
Each extractor will use its MAfuture reply destination to return each new leaf to the comparator.
In this example MAfutures are used as communication channels (as for Unix pipes).

Note that both extraction (by TreeExtractor) and comparison (by TreeComparator) methods are recursive. TreeExtractor method extract: needs to be defined as a private routine because it involves recursion and synchronization. Therefore it cannot be defined in a straightforward way as a script method. (Remember that recursion and synchronous message sending leads to deadlock, thus we would have to find alternative non trivial way of writing down the extraction process).

In case of failure (matching fails) the comparator send messages in the express mode to both extractors in order to stop them (and abort their current tree extraction by using the nonResume construct).'!


!TreeComparator methodsFor: 'initialize'!

initialize
	"Set the two tree extractors."

	extractor1 := (TreeExtractor new number: 1) actor.
	extractor2 := (TreeExtractor new number: 2) actor! !

!TreeComparator methodsFor: 'script'!

sameFringe: tree1 with: tree2
	"Compute a same fringe.
	Initiate the two communication channels through the two MAfuture type (MA for multiple answers) messages sent.
	Then start peeking into the two MAFuture objects used as two communication pipes."

	input1 := extractor1 fringe: tree1 replyTo: #MAfuture.
	input2 := extractor2 fringe: tree2 replyTo: #MAfuture.
	self compareNextElements! !

!TreeComparator methodsFor: 'private routines'!

compareNextElements
	"Compare elements one by one taken from the two MAFutures."

	| element1 |
	element1 := input1 next.
	element1 = (input2 next)
		ifTrue: [element1 = #EOT			"EOT = end of tree special symbol."
					ifTrue: [self success]
					ifFalse: [self compareNextElements]]
		ifFalse: [self failure]!

failure

	Print reply: false.
	extractor1 ExpressStop.		"Stop and abort the two extractors with an express message."
	extractor2 ExpressStop!

success

	Print reply: true! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TreeComparator class
	instanceVariableNames: ''!


!TreeComparator class methodsFor: 'instance creation'!

new

	^super new initialize! !

!TreeComparator class methodsFor: 'example'!

counterExample
	"TreeComparator counterExample"

	TreeComparator new actor sameFringe: #(1 (2 ((3) 40) 5) (6) (((7) 8) 9) ((10)))
							with:		#(1 2 3 4 5 6 7 8 9 10)!

example
	"TreeComparator example"

	TreeComparator new actor sameFringe: #(1 (2 ((3) 4) 5) (6) (((7) 8) 9) ((10)))
							with:		#(1 2 3 4 5 6 7 8 9 10)! !

AbclActorBehavior subclass: #AbclBoundedBuffer
	instanceVariableNames: 'contents maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
AbclBoundedBuffer comment:
'Class AbclBoundedBuffer implements the bounded buffer example.
If a buffer accepted a disabled request (e.g., a put: message once it is full), it will explicitly wait for an alternative request (e.g., a get request) in order to first free the buffer.

See definition of put: and getAndReplyTo: methods.
Note that as opposed to behavior/state replacement as in class EmptyBoundedBuffer (category Actalk-Ext-Replace-Examples), synchronization code appears explicitly within the method body. This is because acceptance of method is controlled after already accepting a disabled request.
Note that mixing up synchronization code with method code is not highly modular. This proves to be a problem for reusing specifications in subclasses.
See discussion of inheritance anomalies in examples of categories Actalk-Synchro-*.

Event handling methods associated to waitFor events are defined for tracing purposes.'!


!AbclBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger

	maxSize := anInteger.
	contents := BoundedBufferObject new maxSize: maxSize! !

!AbclBoundedBuffer methodsFor: 'script'!

getAndReplyTo: r

	| item |
	contents isEmpty							"If empty,"
		ifTrue: [self waitFor: #(put:)				"then, wait for a put:,"
					andDo: [:newItem |
							r reply: newItem]]	"and return directly the new item."
		ifFalse:								"Otherwise,"
			[item := contents removeFirst.		"proceed as usual."
			r reply: item]!

put: item

	contents isFull										"If full,"
		ifTrue: [self waitFor: #(getAndReplyTo:)				"then, wait for a get,"
					andDo: [:r |
							r reply: contents removeFirst]].	"and serve it."
	contents addLast: item			"Here we may assume the buffer is not (or no more) full."! !

!AbclBoundedBuffer methodsFor: 'printing'!

printOn: aStream
	"Print as its class name, then its contents."

	super printOn: aStream.
	contents printOn: aStream! !

!AbclBoundedBuffer methodsFor: 'events'!

eventWaitFor: arrayOfSelectors
	"Trace when entering a waitFor."

	Transcript show: self printString , ' waitFor ' , arrayOfSelectors printString; cr!

eventWaitForAccept: aMessage
	"Trace when accepting a message waitedFor."

	Transcript show: self printString , ' waitForAccept (' , aMessage compactPrintString , ')'; cr!

eventWaitForComplete: aMessage
	"Trace when completing a message waitedFor."

	Transcript show: self printString , ' waitForComplete (' , aMessage compactPrintString , ')'; cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbclBoundedBuffer class
	instanceVariableNames: ''!


!AbclBoundedBuffer class methodsFor: 'instance creation'!

new: maxSize

	^self new initialize: maxSize! !

!AbclBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"AbclBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 4"
	"AbclBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (AbclBoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runNowGet: numberItems! !
