IRActorBehavior subclass: #AcceptActorBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Accept'!
AcceptActorBehavior comment:
'Class AcceptActorBehavior implements actor behaviors with explicit acceptance of messages.
The behavior may explicitly specify within a method body that it is looking for a certain set of messages to accept.
Method answer: accepts a message matching among a set (array) of selectors.
(It is named answer: as in the POOL language, see class PoolActorBehavior comment).
Note that construct answer: is equivalent to construct waitFor:andDo: from class AbclActorBehavior except in that it does not specify any specific computation to be performed, but rather performs the standard method associated to the message accepted.
Method answer is blocking (until it found a matching message). Method unBlockingAnswer: is not, and returns nil if no matching message has been found.
Note that method answer: return the value of the computation of the message. Method unBlockingAnswer: return nil if no matching message may be found.

Class AcceptActorBehavior is defined as a subclass of class ImplicitReplyBehavior to inherit implicit reply of messages (of future type).

Acknowledgements:
See class PoolActorBehavior comment.'!


!AcceptActorBehavior methodsFor: 'message handling'!

answer: arrayOfSelectors
	"Look into the mailbox for the first message matching one of the selectors,
	and then accept it.
	It is blocking and suspends until it finds any matching one.
	(This means it suspends onto incoming messages as long as necessary).
	Return the value of the computation."

	^self acceptMessage: (aself mailBox firstMessageAmong: arrayOfSelectors)!

unBlockingAnswer: arrayOfSelectors
	"Look into the mailbox for the first message matching one of the selectors,
	and then accept it.
	As opposed to method answer: which is blocking, unBlockingAnswer: is not.
	If a matching message is found, perform it and return the value.
	Otherwise return nil."

	"Warning: be cautious when using unblocking answer statements
	within an endless body loop.
	The process may keep the Processor almost undefinitely.
	You have to insert 'Processor yield' statements or use the time slicing facility."

	| message |
	message := aself mailBox checkFirstMessageAmong: arrayOfSelectors.
	^message isNil
		ifTrue: [nil]
		ifFalse: [self acceptMessage: message]! !
