BodyAcceptActorBehavior subclass: #BodyAcceptBoundedBuffer
	instanceVariableNames: 'contents maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-BodyAccept-Ex'!
BodyAcceptBoundedBuffer comment:
'Class BodyAcceptBoundedBuffer implements the bounded buffer example.
The body specifies which messages it may accept according to its state.
Note that synchronization code is outside of the method bodie, which makes it more modular.
However it is centralized within the body. As a result this model is not very good at reusing specifications because it has to fully redefine the body.
See discussion of inheritance anomalies in examples of categories Actalk-Synchro-*.'!


!BodyAcceptBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger

	maxSize := anInteger.
	contents := BoundedBufferObject new maxSize: maxSize! !

!BodyAcceptBoundedBuffer methodsFor: 'body'!

body
	"Body: change/dispatch acceptance of messages according to the state of the buffer."

	[true] whileTrue:
		[self answer:
			(contents isEmpty
				ifTrue: [#(put:)]
				ifFalse: [contents isFull
							ifTrue: [#(get)]
							ifFalse: [#(put: get)]])]! !

!BodyAcceptBoundedBuffer methodsFor: 'script'!

get

	^contents removeFirst!

put: item

	contents addLast: item! !

!BodyAcceptBoundedBuffer methodsFor: 'printing'!

printOn: aStream
	"Print as its class name, then its contents."

	super printOn: aStream.
	contents printOn: aStream! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BodyAcceptBoundedBuffer class
	instanceVariableNames: ''!


!BodyAcceptBoundedBuffer class methodsFor: 'instance creation'!

new: size

	^self new initialize: size! !

!BodyAcceptBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"BodyAcceptBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 4"
	"BodyAcceptBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (BodyAcceptBoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runGet: numberItems! !

SimulatedBodyAcceptActorBehavior subclass: #SimulatedBodyAcceptBoundedBuffer
	instanceVariableNames: 'contents maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-BodyAccept-Ex'!
SimulatedBodyAcceptBoundedBuffer comment:
'Class SimulatedBodyAcceptBoundedBuffer implements the bounded buffer example.
It is equivalent to class BodyAcceptBoundedBuffer except in that it simulates the body through some initial message (see class SimulatedBodyAcceptActorBehavior).'!


!SimulatedBodyAcceptBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger

	maxSize := anInteger.
	contents := BoundedBufferObject new maxSize: maxSize! !

!SimulatedBodyAcceptBoundedBuffer methodsFor: 'body'!

body
	"Body: change/dispatch acceptance of messages according to the state of the buffer."

	[true] whileTrue:
		[self answer:
			(contents isEmpty
				ifTrue: [#(put:)]
				ifFalse: [contents isFull
							ifTrue: [#(get)]
							ifFalse: [#(put: get)]])]! !

!SimulatedBodyAcceptBoundedBuffer methodsFor: 'script'!

get

	^contents removeFirst!

put: item

	contents addLast: item! !

!SimulatedBodyAcceptBoundedBuffer methodsFor: 'printing'!

printOn: aStream
	"Print as its class name, then its contents."

	super printOn: aStream.
	contents printOn: aStream! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimulatedBodyAcceptBoundedBuffer class
	instanceVariableNames: ''!


!SimulatedBodyAcceptBoundedBuffer class methodsFor: 'instance creation'!

new: size

	^self new initialize: size! !

!SimulatedBodyAcceptBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"SimulatedBodyAcceptBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 4"
	"SimulatedBodyAcceptBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (SimulatedBodyAcceptBoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runGet: numberItems! !
