AcceptActorBehavior subclass: #SimulatedBodyAcceptActorBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-BodyAccept'!
SimulatedBodyAcceptActorBehavior comment:
'Class SimulatedBodyAcceptActorBehavior purpose is only to demonstrate that an active object with a body (autonomous live routine whose computation starts once the object is created) may be immediately simulated by an actor which sends to itself the body as a message as soon as created.
As a result this initial body message will be the first and only message to be implicitly accepted, as next ones will be explicitly accepted from within the body specification.'!


!SimulatedBodyAcceptActorBehavior methodsFor: 'initialize'!

initializeAndSetAself: anActor
	"Initialize the behavior and send to itself the body message to start."

	super initializeAndSetAself: anActor.
	aself body! !

AcceptActorBehavior subclass: #BodyAcceptActorBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-BodyAccept'!
BodyAcceptActorBehavior comment:
'Class AcceptActorBehavior implements actor behaviors with a body.
The body is a live routine where the behavior explicitly specifies which sequence of which set of messages it is willing to accept. (See examples of body use in category Actalk-Ext-BodyAccept-Ex).
Computation of the body starts as soon as the object is created.

Class BodyAcceptActorBehavior is defined as a subclass of class AcceptActorBehavior to inherit explicit acceptance of messages which is the only way to accept messages (because there is no implicit acceptance of messages as for standard actors).

Class BodyAcceptActorBehavior corresponds roughly to POOL body and explicit message acceptance model except that message passing is of future type and not synchronous.
(However derivation of POOL type actors is instant in that they just need to implicitly wait onto the future returned. See classes PoolActor(Behavior)).

Acknowledgements:
See class PoolActorBehavior comment.'!


!BodyAcceptActorBehavior methodsFor: 'process setting'!

behaviorProcessBlock
	"The autonomous process runs the body of the actor."

	^self bodyBehaviorProcessBlock!

bodyBehaviorProcessBlock
	"The body is the specification of explicit acceptance of messages."

	^[self body]! !

!BodyAcceptActorBehavior methodsFor: 'body'!

body
	"The body must be specified in subclasses."

	self subclassResponsability! !
