IRActorBehavior subclass: #Thief
	instanceVariableNames: 'name bag organization '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-ImplicitReply-Ex'!
Thief comment:
'Classes Thief and ThiefOrganization implement an example of thieves actors whose activity is to steal from each other the things they own.
If a thief owns no more things, then it has to give away his life, and consequently kills itself.
Basic activity of a thief is: look for a victim (by asking the organization), request a thing from the victim, sleep a little bit, and do it again.
Because selection of a victim and sleeping time is random, eventually only one thief will remain.

Instance Variables:

	name			<String>						the name of the thief.
	bag				<OrderedCollection>			contains the things owned by the thief.
	organization		<ThiefOrganization actor>		the organization registering all thieves.'!


!Thief methodsFor: 'initialize'!

name: aString owns: arrayOfThings organization: aThiefOrganization

	name := aString.
	bag := arrayOfThings asOrderedCollection.
	organization := aThiefOrganization! !

!Thief methodsFor: 'script'!

giveSomethingAndReplyTo: r

	bag isEmpty
		ifTrue:		"If no more thing to give, then the thief kills itself."
			[organization unregister: aself.
			Transcript show: self printString , ' is killed by ' , r printString; cr.
			self terminate]
		ifFalse:		"Otherwise, it replies one thing among the ones it owns."
			[r reply: bag removeOneRandom]!

reply: aThing

	bag add: aThing!

run
	"Main activity for a thief."

	| victim |
	victim := (organization oneExcept: aself) value.	"Look for a victim."
	victim isNil
		ifTrue:		"If no victim can be found, this means that the thief is the only remaining."
			[Transcript show: self printString , ' is a total winner!!'; cr]
		ifFalse:		"Otherwise, it orders its victim to give one thing,"
			[victim giveSomethingAndReplyTo: aself.
			(Delay forMilliseconds: (Random new next * 1000) rounded) wait.	"sleeps a little bit,"
			Transcript show: name , '(' , bag size printString , ') '.
			aself run]	"and does it again."! !

!Thief methodsFor: 'printing'!

printOn: aStream

	aStream nextPutAll: name! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Thief class
	instanceVariableNames: ''!


!Thief class methodsFor: 'example'!

example
	"Thief example"

	| mafia |
	mafia := ThiefOrganization new actor.
	mafia
		register: (Thief new name: 'Jeeps' owns: #(PowerBook bass bicycle stereo) organization: mafia) actor;
		register: (Thief new name: 'Jeff' owns: #(scuba PC bike Tshirt) organization: mafia) actor;
		register: (Thief new name: 'Laurent' owns: #(coffeeMachine kanjiBook camera Self) organization: mafia) actor;
		register: (Thief new name: 'Jacques' owns: #(KoreanDictionary lambda spices garlic) organization: mafia) actor;
		register: (Thief new name: 'Gao' owns: #(alphaBeta transputer neckTie glasses) organization: mafia) actor.
	mafia start! !

IRActorBehavior subclass: #IRServer
	instanceVariableNames: 'contents '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-ImplicitReply-Ex'!
IRServer comment:
'Class IRServer shows a very simple (and not very interesting!!) example of use of implicit reply actors. The benefit is in preserving standard Smalltalk way of writing and using methods.'!


!IRServer methodsFor: 'script'!

computeDuring: time

	Transcript show: '(server computes for ' , time printString , ' seconds '.
	time timesRepeat:
		[(Delay forSeconds: 1) wait.
		Transcript show: '.'].
	Transcript show: ') '.
	^time	"time is the returned value."! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IRServer class
	instanceVariableNames: ''!


!IRServer class methodsFor: 'example'!

example
	"IRServer example"

	"As for standard Smalltalk programs except that the returned value is a future."
	"Therefore we need to add an explicit value fetch."

	| n |
	n := (IRServer new actor computeDuring: 5) value.	"Standard way of message passing."
	Transcript show: 'n = ' , n printString; cr!

example2
	"IRServer example2"

	"As for standard Smalltalk programs except that the returned value is a future."
	"In this example, we show the synchronization taking place."

	| n |
	n := IRServer new actor computeDuring: 5.	"Standard way of message passing."
	Transcript show: 'n = ' , n printString; cr.					"The future will be printed."
	Transcript show:'n+1 = '; show: (n + 1) printString; cr		"This time we need to wait for the value."! !

IRActorBehavior subclass: #ThiefOrganization
	instanceVariableNames: 'thieves '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-ImplicitReply-Ex'!
ThiefOrganization comment:
'Class ThiefOrganization implements an organization of thieves where they can register (at start), unregister (when killed), and ask for a victim (one of the thieves).

Instance Variables:

	thieves		<OrderedCollection>		all registered thieves.

See comment of class Thief.'!


!ThiefOrganization methodsFor: 'initialize'!

initialize

	thieves := OrderedCollection new! !

!ThiefOrganization methodsFor: 'script'!

oneExcept: aThieve
	"Return a thief (victim) requested.
	It cannot be the requester. Return nil if only one thief is remaining."

	^thieves size = 1
		ifTrue: [nil]
		ifFalse: [thieves oneRandomExcept: aThieve]!

register: aThieve

	thieves add: aThieve!

start

	thieves do: [:thief | thief run]!

unregister: aThieve

	thieves remove: aThieve! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ThiefOrganization class
	instanceVariableNames: ''!


!ThiefOrganization class methodsFor: 'instance creation'!

new

	^super new initialize! !
