ReplaceActorBehavior subclass: #ReplaceBinaryAdder
	instanceVariableNames: 'r '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Replace-JoinCont'!
ReplaceBinaryAdder comment:
'Class ReplaceBinaryAdder represents a join continuation which adds two values along the behavior replacement model (class ReplaceActorBehavior).
Its behavior replacement is a ReplaceUnaryAdder which stores the first value received.
We assume that the operation (+) is commutative.

Instance Variables:

	r	<Actor>	the reply destination to which the computed result has to be sent.'!


!ReplaceBinaryAdder methodsFor: 'initialize'!

r: aReplyDestination
	r := aReplyDestination! !

!ReplaceBinaryAdder methodsFor: 'script'!

reply: v
	self replace: (ReplaceUnaryAdder new v1: v r: r)! !

ReplaceActorBehavior subclass: #ReplaceUnaryAdder
	instanceVariableNames: 'v1 r '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Replace-JoinCont'!
ReplaceUnaryAdder comment:
'Class ReplaceUnaryAdder represents a join continuation which adds two values and has already received one value. It simply adds up the previously stored value (by ReplaceBinaryAdder) to the incoming one and replies the result.

Note that no replacement behavior is specified as no more message is expected. Computation terminates.
We could alternatively specify a NoMoreMessage behavior as the replacement behavior, but it would create and maintain a process resource.

Instance Variables:

	v1	<Integer>	the first value received.
	r	<Actor>		the reply destination to which the computed result has to be sent.'!


!ReplaceUnaryAdder methodsFor: 'initialize'!

v1: anInteger r: aReplyDestination
	v1 := anInteger.
	r := aReplyDestination! !

!ReplaceUnaryAdder methodsFor: 'script'!

reply: v2
	r reply: v1 + v2! !
