ASCActorBehavior subclass: #ASCBoundedBuffer
	instanceVariableNames: 'array getIndex putIndex maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-ASC-Examples'!


!ASCBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger

	maxSize := anInteger.
	array := Array new: maxSize + 1.
	getIndex := 1.
	putIndex := 1! !

!ASCBoundedBuffer methodsFor: 'abstract states'!

empty

	^#(put:)!

full

	^#(get)!

initialAbstractState

	^#empty!

partial

	^(self empty) + (self full)! !

!ASCBoundedBuffer methodsFor: 'state transition'!

nextAbstractStateAfter: selector

	^self isEmpty
		ifTrue: [#empty]
		ifFalse: [self isFull
					ifTrue: [#full]
					ifFalse: [#partial]]! !

!ASCBoundedBuffer methodsFor: 'script'!

get

	| item |
	item := array at: getIndex.
	getIndex := (getIndex \\ array size) + 1.
	(Delay forSeconds: 2) wait.
	^item!

put: item

	array at: putIndex put: item.
	putIndex := (putIndex \\ array size) + 1.
	(Delay forSeconds: 2) wait! !

!ASCBoundedBuffer methodsFor: 'guards'!

guardOFget
	"Only one get at once."

	^(self current: #get) = 0!

guardOFput: item
	"Only one put at once."

	^(self current: #put:) = 0! !

!ASCBoundedBuffer methodsFor: 'state predicates'!

isEmpty

	^getIndex = putIndex!

isFull

	^(putIndex \\ array size) = (getIndex - 1)! !

!ASCBoundedBuffer methodsFor: 'printing'!

printOn: aStream

	super printOn: aStream.
	aStream nextPut: $(.
	getIndex <= putIndex
		ifTrue:
			[getIndex to: putIndex - 1 do:
				[:i |
				(array at: i) printOn: aStream.
				aStream nextPut: $ ]]
		ifFalse:
			[getIndex to: array size do:
				[:i |
				(array at: i) printOn: aStream.
				aStream nextPut: $ ].
			1 to: putIndex -1 do:
				[:i |
				(array at: i) printOn: aStream.
				aStream nextPut: $ ]].
	aStream nextPut: $)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ASCBoundedBuffer class
	instanceVariableNames: ''!


!ASCBoundedBuffer class methodsFor: 'instance creation'!

new: size
	^self new initialize: size! !

!ASCBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"ASCBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (ASCBoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 0) actor.
	consumer := (Consumer new buffer: buffer delay: 2) actor.
	producer runPut: numberItems.
	consumer runGet: numberItems! !

ASCBoundedBuffer subclass: #ASCGetRearBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-ASC-Examples'!


!ASCGetRearBoundedBuffer methodsFor: 'abstract states'!

full
	^(super full) , #(getRear)! !

!ASCGetRearBoundedBuffer methodsFor: 'script'!

getRear
	putIndex := ((putIndex - 2) \\ (array size)) + 1.
	^array at: putIndex! !

!ASCGetRearBoundedBuffer methodsFor: 'guards'!

guardOFget
	"."
	^super guardOFget
		and: [(self current: #getRear) = 0]!

guardOFgetRear
	"."
	^(self current: #getRear) = 0
		and: [(self current: #get) = 0
				and: [(self current: #put:) = 0]]!

guardOFput: item
	"."
	^(super guardOFput: item)
		and: [(self current: #getRear) = 0]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ASCGetRearBoundedBuffer class
	instanceVariableNames: ''!


!ASCGetRearBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"ASCGetRearBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 1"

	| buffer producer consumer |
	buffer := (ASCGetRearBoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runGetRear: numberItems! !

ASCBoundedBuffer subclass: #ASCGetRear2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-ASC-Examples'!


!ASCGetRear2BoundedBuffer methodsFor: 'abstract states'!

full
	^(super full) , #(getRear)!

one
	^(self empty) , #(get)! !

!ASCGetRear2BoundedBuffer methodsFor: 'state transition'!

nextAbstractStateAfter: selector

	^self isOne
		ifTrue: [#one]
		ifFalse: [super nextAbstractStateAfter: selector]! !

!ASCGetRear2BoundedBuffer methodsFor: 'script'!

getRear
	putIndex := ((putIndex - 2) \\ (array size)) + 1.
	^array at: putIndex! !

!ASCGetRear2BoundedBuffer methodsFor: 'guards'!

guardOFgetRear
	"."
	^self guardOFput: #dummy!

guardOFput: item
	"."
	^(super guardOFput: item)
		and: [(self current: #getRear) = 0]! !

!ASCGetRear2BoundedBuffer methodsFor: 'state predicate'!

isOne
	^(putIndex - 1) = (getIndex \\ array size)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ASCGetRear2BoundedBuffer class
	instanceVariableNames: ''!


!ASCGetRear2BoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"ASCGetRear2BoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 1"

	| buffer producer consumer |
	buffer := (ASCGetRear2BoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runGetRear: numberItems! !

ASCBoundedBuffer subclass: #ASCGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-ASC-Examples'!


!ASCGet2BoundedBuffer methodsFor: 'abstract states'!

full
	^(super full) , #(get2)!

one
	^(self empty) , #(get)! !

!ASCGet2BoundedBuffer methodsFor: 'state transition'!

nextAbstractStateAfter: selector

	^self isOne
		ifTrue: [#one]
		ifFalse: [super nextAbstractStateAfter: selector]! !

!ASCGet2BoundedBuffer methodsFor: 'script'!

get2
	^Array with: self get with: self get! !

!ASCGet2BoundedBuffer methodsFor: 'guards'!

guardOFget
	"."
	^super guardOFget
		and: [(self current: #get2) = 0]!

guardOFget2
	"."
	^self guardOFget! !

!ASCGet2BoundedBuffer methodsFor: 'state predicates'!

isOne
	^(putIndex - 1) = (getIndex \\ array size)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ASCGet2BoundedBuffer class
	instanceVariableNames: ''!


!ASCGet2BoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"ASCGet2BoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (ASCGet2BoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: ratio) actor.
	consumer := (Consumer new buffer: buffer delay: 1) actor.
	producer runPut: numberItems.
	consumer runGet2: numberItems! !
