ConcurrentActorBehavior subclass: #CountersActorBehavior
	instanceVariableNames: 'receivedDictionary acceptedDictionary completedDictionary '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Counters'!
CountersActorBehavior comment:
'This class has been initially designed and implemented by the DESS class at Universite de Nantes, advised by Jean Bezivin and Olivier Roux. This is a revised version.'!


!CountersActorBehavior methodsFor: 'initialize'!

actalkPrivateInitialize
	"Create the three synchronization counters dictionaries."

	super actalkPrivateInitialize.
	receivedDictionary := IdentityDictionary new.
	acceptedDictionary := IdentityDictionary new.
	completedDictionary := IdentityDictionary new.
	self class allScriptSelectors
		do: [:selector |
			receivedDictionary at: selector put: 0.
			acceptedDictionary at: selector put: 0.
			completedDictionary at: selector put: 0]! !

!CountersActorBehavior methodsFor: 'message handling'!

acceptMessage: aMessage
	"Accept a message by first checking the guard.
	If the guard evaluates to true, then increment the accept counter
	and launch a behavior sub process to consume the message.
	If the guard evaluates to false, resend the message."
	"Note that checking the guard AND incrementing the accept counter
	(if the guard evaluates to true) must be within the same atomic block.
	(Consequently we don't make use of the accept generic synchro event
	to increment the accept counter.)"

	synchroMutualExclusion critical:
		[(self satisfyGuardSelector: aMessage selector arguments: aMessage arguments)
			ifTrue:
				[self incrAccepted: aMessage selector.
				self setSubProcessFor: aMessage]
			ifFalse:
				[aself resend: aMessage]]! !

!CountersActorBehavior methodsFor: 'subprocess setting'!

behaviorSubProcessBlockFor: aMessage
	"This is the behavior process block for a subprocess computing a new message.
	Accept and consume a message."

	^[self subProcessAcceptMessage: aMessage]

	"Generic event completed message takes care of updating associated synchro counter."! !

!CountersActorBehavior methodsFor: 'guard testing'!

findGuardSelector: selector

	^('guardOF' , selector) asSymbol!

satisfyGuardSelector: selector arguments: argumentsArray
	"Check guard for message selector."

	^self perform: (self findGuardSelector: selector) withArguments: argumentsArray! !

!CountersActorBehavior methodsFor: 'synchro events'!

synchroComplete: aMessage

	self incrCompleted: aMessage selector!

synchroReceive: aMessage

	self incrReceived: aMessage selector! !

!CountersActorBehavior methodsFor: 'counters accessing'!

accepted: selector
	"Number of activations accepted for message selector."

	^acceptedDictionary at: selector!

completed: selector
	"Number of activations completed for message selector."

	^completedDictionary at: selector!

current: selector
	"Number of current activations for message selector."

	^(self accepted: selector) - (self completed: selector)!

pending: selector
	"Number of pending activations for message selector."

	^(self received: selector) - (self accepted: selector)!

received: selector
	"Number of requests received for message selector."

	^receivedDictionary at: selector! !

!CountersActorBehavior methodsFor: 'counters updating'!

incrAccepted: selector
	"Increment the number of activations accepted for message selector."

	acceptedDictionary at: selector
					put: (acceptedDictionary at: selector) + 1!

incrCompleted: selector
	"Increment the number of activations completed for message selector."

	completedDictionary at: selector
					put: (completedDictionary at: selector) + 1!

incrReceived: selector
	"Increment the number of messages received for message selector."

	receivedDictionary at: selector
					put: (receivedDictionary at: selector) + 1! !
